@@ -1,6 +1,6 @@
 { 
 package Audio::Nama::Effect;
-use Modern::Perl;
+use Modern::Perl '2020';
 use List::MoreUtils qw(first_index insert_after_string);
 use Carp qw(carp cluck croak confess);
 use Data::Dumper::Concise;
@@ -34,11 +34,9 @@ use Audio::Nama::Object qw(
 				bypassed
 				name
 				surname 
+				predecessor
 
 );
-*this_op			= \&Audio::Nama::this_op;
-*this_param			= \&Audio::Nama::this_param;
-*this_stepsize		= \&Audio::Nama::this_stepsize;
 our %by_id;
 our $AUTOLOAD;
 import_engine_subs();
@@ -56,7 +54,7 @@ sub initialize {
 sub AUTOLOAD {
 	my $self = shift;
 	#say "got self: $self", Audio::Nama::Dumper $self;
-	die 'not object' unless ref $self;
+	confess 'not object' unless ref $self;
 	# get tail of method call
 	my ($call) = $AUTOLOAD =~ /([^:]+)$/;
 	# see if this can be satisfied by a field from
@@ -78,28 +76,25 @@ sub new {
 
 	my $id = $args{id};
 
-	# return existing object if effect already exists
-	if ($self = fxn($id)){
-		logpkg(__FILE__,__LINE__,'debug',"$id: returning existing object"); 
+	# return an existing object that has this ID
+	# Why do this instead of throw an exception?
+	if ($id and $self = fxn($id)){
+		logpkg(__FILE__,__LINE__,'debug',"$id: returning existing object, constructor arguments ignored"); 
 		return $self
 	}
 
-	# allocate effect ID
-	my	$how_allocated = "recycled";
-	if ( ! $id ){ 
-		$id = new_effect_id();
-		$how_allocated = "issued";
-	}
-	logpkg(__FILE__,__LINE__,'debug',"$id: effect id $how_allocated");
-
 	my $i = effect_index($args{type});
-	defined $i or confess "$args{type}: effect index not found.";
+	defined $i or confess "$args{type}: plugin not found.";
 
-	logpkg(__FILE__,__LINE__,'debug',"$id: Issuing effect id for track $args{chain}");
+	# allocate effect ID if we don't have one yet
+	#
+	my	$how_allocated = $id ? 'recycled' : 'issued';
+	$id //= new_effect_id();
+	logpkg(__FILE__,__LINE__,'debug',"$id: effect ID $how_allocated for track $args{chain}");
 	
 	$args{id}		= $id;
-	$args{display} 	= $fx_cache->{registry}->[$i]->{display};
-	$args{owns}		= [];
+	$args{display} 	//= $fx_cache->{registry}->[$i]->{display};
+	$args{owns}		//= [];
 
 	my $track = $ti{$args{chain}};
 
@@ -143,11 +138,9 @@ sub new {
 		logpkg(__FILE__,__LINE__,'debug',"parent owns @$owns");
 
 		# register effect_id with parent unless it is already there
-		if (! grep { $id eq $_ } @$owns) {
-			push @$owns, $id;
-			logpkg(__FILE__,__LINE__,'debug',sub{join " ", "my attributes:", json_out($self->as_hash)});
-		}
-		logpkg(__FILE__,__LINE__,'debug',sub{join " ", "my attributes again:", json_out($self->as_hash)});
+		push @$owns, $id unless grep { $id eq $_ } @$owns;
+
+		logpkg(__FILE__,__LINE__,'debug',sub{join " ", "my attributes:", json_out($self->as_hash)});
 		# find position of parent id in the track ops array 
  		# and insert child id immediately afterwards
  		# unless already present
@@ -162,10 +155,14 @@ sub new {
 	} 
 	$self
 }
-
-# fx method delivers hash, previously via $fx->{ applied}->{$id}
-# TODO: get rid of this entirely
-sub fx	 		{ my $self = shift; $self }	
+# provide object
+{
+no warnings 'redefine';
+sub parent {
+	my $self = shift;
+	fxn($self->{parent});
+}
+}
 
 sub is_read_only {
     my ($self, $param) = @_;
@@ -176,6 +173,8 @@ sub remove_name { my $self = shift; delete $self->{name} }
 sub set_name    { my $self = shift; $self->{name} = shift }
 sub set_surname { my $self = shift; $self->{surname} = shift}
 sub is_controller { my $self = shift; $self->parent } 
+sub is_channel_op { my $self = shift; $config->{ecasound_channel_ops}->{$self->type} }
+
 
 sub has_read_only_param {
 	my $self = shift;
@@ -191,65 +190,67 @@ sub registry_index {
 	my $self = shift;
 	$fx_cache->{full_label_to_index}->{ $self->type };
 }
-sub ecasound_controller_index {
-	logsub("&ecasound_controller_index");
-	my $self = shift;
-	my $id = $self->id;
-	my $chain = $self->chain;
-	my $track = $ti{$chain};
-	my @ops = @{$track->ops};
-	my $operator_count = 0;
-	my $position;
-	for my $i (0..scalar @ops - 1) {
-		$position = $i, last if $ops[$i] eq $id;
-		$operator_count++ if ! Audio::Nama::fxn($ops[$i])->is_controller;
-	}
-	$position -= $operator_count; # skip operators
-	++$position; # translates 0th to chain-position 1
-}
-sub ecasound_operator_index { # does not include offset
-	logsub("&ecasound_operator_index");
-	my $self = shift;
-	my $id = $self->id;
-	my $chain = $self->chain;
-	my $track = $ti{$chain};
-	my @ops = @{$track->ops};
-	my $controller_count = 0;
-	my $position;
-	for my $i (0..scalar @ops - 1) {
-		$position = $i, last if $ops[$i] eq $id;
-		$controller_count++ if Audio::Nama::fxn($ops[$i])->is_controller;
-	}
-	$position -= $controller_count; # skip controllers 
-	++$position; # translates 0th to chain-position 1
-}
-sub ecasound_effect_index { 
-	logsub("&ecasound_effect_index");
+sub ecasound_controller_index { 
+	logsub((caller(0))[3]);
 	my $self = shift;
 	my $n = $self->chain;
 	my $id = $self->id;
 	my $opcount = 0;
 	logpkg(__FILE__,__LINE__,'debug', "id: $id, n: $n, ops: @{ $ti{$n}->ops }" );
 	for my $op (@{ $ti{$n}->ops }) { 
-			# increment only for ops, not controllers
-			next if $self->is_controller;
+		# increment only controllers
+		next unless fxn($op)->is_controller;
+		$opcount++;
+		last if $op eq $id;
+	} 
+	$opcount;
+}
+sub ecasound_effect_index { 
+	logsub((caller(0))[3]);
+	my $self = shift;
+	my $n = $self->chain;
+	my $id = $self->id;
+	my $opcount = 0;
+	logpkg(__FILE__,__LINE__,'debug', "id: $id, n: $n, ops: @{ $ti{$n}->ops }" );
+	for my $op (@{ $ti{$n}->ops }) { 
+			my $fx = fxn($op);
+ 			next if $fx->is_controller;
 			++$opcount;   # first index is 1
 			last if $op eq $id
 	} 
 	no warnings 'uninitialized';
 	$self->offset + $opcount;
 }
+sub ecasound_effect_index_ {
+	my $self = shift;
+	1 + first_index {$_->id eq $self->id } $self->track->ops_ecasound_order();
+}
 sub track_effect_index { # the position of the ID in the track's op array
 	my $self = shift;
 	my $id = $self->id;
 	my $pos = first_index {$id eq $_} @{$self->track->ops} ;
 	$pos
 }
+sub controllers {
+	my $self = shift;
+	my %children;
+	# we want controllers with this parent, also controllers
+	# whos parents are children of this parent, and children
+	# of those children
+	no warnings;
+	my @ctrl =	map { $_->id }
+				grep{ $_->{parent} eq $self->id 
+					or $children{$_->{parent}} 
+					and $children{$_->id}++ 
+					} map{ fxn($_)} @{ $self->track->ops };
+
+	@ctrl
+}
 sub sync_one_effect {
-		my $self= shift;
+		my $self = shift;
 		my $chain = $self->chain;
-		Audio::Nama::eval_iam("c-select $chain");
-		Audio::Nama::eval_iam("cop-select " .( $self->offset + $self->ecasound_operator_index ) );
+		$this_engine->current_chain($chain);
+		$this_engine->current_chain_operator($self->ecasound_effect_index);
 		$self->set(params => get_ecasound_cop_params( scalar @{$self->params} ));
 }
 sub offset {
@@ -258,8 +259,8 @@ sub offset {
 }
 sub root_parent { 
 	my $self = shift;
-	return $self if ! $self->parent;
-	$self->parent->root_parent
+	my $parent = $self->parent;
+	$parent and $parent->parent or $parent or $self
 }
 sub about {
 	my $self = shift;
@@ -270,14 +271,19 @@ sub trackname { $_[0]->track->name }
 
 sub ladspa_id {
 	my $self = shift;
-	$Audio::Nama::fx_cache->{ladspa_label_to_unique_id}->{$self->type} 
+	$fx_cache->{ladspa_label_to_unique_id}->{$self->type} 
 }
 sub nameline {
 	my $self = shift;
-	my @attr_keys = qw( name surname fxname type ladspa_id bypassed trackname);
-	my $nameline = $self->id. ": ". join q(, ), grep{$_} map{$self->$_} @attr_keys;
-	$nameline .= "\n";
-	$nameline
+	my @attr_keys = qw(id name surname fxname type ladspa_id);
+	my %display = map{ $_ => 1         } grep { !/fxname|type|ladspa_id|id/ } @attr_keys;
+	my %attr    = map{ $_ => $self->$_ } @attr_keys ;
+	my $bypassed = $self->{bypassed} ? " (bypassed)" : '';
+	not defined $attr{$_} and delete $attr{$_} for @attr_keys;
+	my $nameline =  join qq( ),  
+			map{ $display{$_} ? "$_:$attr{$_}" : $attr{$_} } grep{$attr{$_}}  @attr_keys;
+	$nameline .= "$bypassed\n";
+	$nameline;
 }
 sub _effect_index { 
 	my $self = shift;
@@ -315,7 +321,7 @@ sub _modify_effect {
 	1
 }
 sub _remove_effect { 
-	logsub("&_remove_effect");
+	logsub((caller(0))[3]);
 	my $self = shift;
 	my $id = $self->id;
 	my $n 		= $self->chain;
@@ -356,14 +362,8 @@ sub _remove_effect {
 	
 	if( my $track = $ti{$n} ){
 		my @ops_list = @{$track->ops};
-		#say "ops_list: @ops_list";
-		my $perl_version = $^V;
-		my ($minor_version) = $perl_version =~ /^v5\.(\d+)/;
 		my @new_list = grep  { $_ ne $id  } @ops_list;
-		#say "new_list: @new_list";
-		if ($minor_version <= 14) 
-		     {    $track->{ops}   = [ @new_list  ] }
-		else { @{ $track->{ops} } =   @new_list    }
+		$track->{ops} =   [ @new_list ];
 	}
 	#set_current_op($this_track->ops->[0]);
 	#set_current_param(1);
@@ -371,12 +371,11 @@ sub _remove_effect {
 	return(); 
 }
 sub position_effect {
+	#logsub((caller(0))[3]);
 	my($self, $pos) = @_;
 
 	my $op = $self->id;
 	
-	# disabled, debugging needed
-	# we cannot handle controllers
 	#Audio::Nama::pager("$op or $pos: controller not allowed, skipping.\n"), return 
 	#	if grep{ fxn($_)->is_controller } $op, $pos;
 	
@@ -385,14 +384,16 @@ sub position_effect {
 	my $track = $ti{$self->chain};
 
 	my $op_index = $self->track_effect_index;
+	my @children = $self->controllers;
+	my $count = scalar @children + 1;
 	my @new_op_list = @{$track->ops};
 
-	# remove op
-	splice @new_op_list, $op_index, 1;
+	# remove op and children
+	my @op_and_ctrl = splice @new_op_list, $op_index, $count;
 
 	if ( $pos eq 'ZZZ'){
 		# put it at the end
-		push @new_op_list, $op;
+		push @new_op_list, @op_and_ctrl
 	}
 	else { 
 		my $POS = fxn($pos);
@@ -401,19 +402,20 @@ sub position_effect {
 			unless $track eq $track2;
 		my $new_op_index = $POS->track_effect_index; 
 		# insert op
-		splice @new_op_list, $new_op_index, 0, $op;
+		splice @new_op_list, $new_op_index, 0, @op_and_ctrl;
+
 	}
-	# reconfigure the entire engine (inefficient, but easy to do)
+	# easier to reconfigure the engine than to code for
+	# repositioning ecasound effects.
 	say join " - ",@new_op_list;
 	@{$track->ops} = @new_op_list;
 	Audio::Nama::request_setup();
 	$this_track = $track;
-	# this command generates spurious warnings during test
-	process_command('show_track');
+	nama_cmd('show_track');
 }
 
 sub apply_op {
-	logsub("&apply_op");
+	logsub((caller(0))[3]);
 	my $self = shift;
 	local $config->{category} = 'ECI_FX';
 	my $id = $self->id;
@@ -442,10 +444,10 @@ sub apply_op {
 
 	logpkg(__FILE__,__LINE__,'debug', "command: $add_cmd");
 
-	Audio::Nama::eval_iam("c-select $chain"); 
-	Audio::Nama::eval_iam("cop-select " . $dad->ecasound_effect_index) if $dad;
-	Audio::Nama::eval_iam($add_cmd);
-	Audio::Nama::eval_iam("cop-bypass on") if $self->bypassed;
+	$this_engine->current_chain($chain); 
+	$this_engine->current_chain_operator($dad->ecasound_effect_index) if $dad;
+	$this_engine->ecasound_iam($add_cmd);
+	$this_engine->ecasound_iam("cop-bypass on") if $self->bypassed;
 
 	my $owns = $self->owns;
 	(ref $owns) =~ /ARRAY/ or croak "expected array";
@@ -455,18 +457,15 @@ sub apply_op {
 
 #### Effect related routines, some exported, non-OO
 
+# main namespace imports from us, we'll import manually
+# to work around dependence issues
+
 sub import_engine_subs {
 
-	*valid_engine_setup = \&Audio::Nama::valid_engine_setup;
-	*engine_running		= \&Audio::Nama::engine_running;
-	*eval_iam			= \&Audio::Nama::eval_iam;
-	*ecasound_select_chain = \&Audio::Nama::ecasound_select_chain;
+	*ecasound_iam			= \&Audio::Nama::ecasound_iam;
 	*sleeper			= \&Audio::Nama::sleeper;
-	*process_command    = \&Audio::Nama::process_command;
+	*nama_cmd    = \&Audio::Nama::nama_cmd;
 	*pager				= \&Audio::Nama::pager;
-	*this_op			= \&Audio::Nama::this_op;
-	*this_param			= \&Audio::Nama::this_param;
-	*this_stepsize		= \&Audio::Nama::this_stepsize;
 }
 
 use Exporter qw(import);
@@ -487,7 +486,7 @@ our %EXPORT_TAGS = ( 'all' => [ qw(
 					modify_effect
 					modify_multiple_effects
 
-					_update_effect
+					update_ecasound_effect
 					update_effect
 					sync_effect_parameters
 					find_op_offsets
@@ -502,10 +501,17 @@ our %EXPORT_TAGS = ( 'all' => [ qw(
 
 					set_current_op
 					set_current_param
-					set_current_stepsize
+					this_param
+					this_op
+					this_op_o
+					param_stepsize
+					set_param_value
+
 					increment_param
-					decrement_param
-					set_parameter_value
+					increment_param_1
+					increment_param_10
+					decrement_param_1
+					decrement_param_10
 
 ) ] );
 
@@ -563,14 +569,24 @@ sub set_chain_value {
 #				* restores the operators
 		 
 sub add_effect {
-	#logsub('&add_effect');
+	#logsub((caller(0))[3]);
 	my $args = shift;
 	my $added = _add_effect($args);
+	initialize_current_param_and_stepsize(@$added);
 	$added->[0]->id
 }
+sub initialize_current_param_and_stepsize {
+	for my $FX (@_){
+		my $param_count = $FX->about->{count};
+		$project->{current_param}->{$FX->id} //=  $param_count ? 1 : 0;
+		for (1..$param_count){
+			$project->{param_stepsize}->{$FX->id}->{$_} //= 1;
+		}
+	}
+}
 sub _add_effect {
 	my $p = shift;
-	logsub("&_add_effect");
+	logsub((caller(0))[3]);
 	#logpkg(__FILE__,__LINE__,'debug',sub{ "add effect arguments - 0:\n".json_out($p)});
 	
 	set_chain_value($p);
@@ -629,6 +645,7 @@ sub append_effect {
 			}  
 		}
 		my $FX = Audio::Nama::Effect->new(%args);
+		$args{self} = $FX;
 		push @added, $FX;
 		if( ! $FX->name )
 		{
@@ -641,11 +658,12 @@ sub append_effect {
 		}
 		$ui->add_effect_gui(\%args) unless $track->hide;
 
-		$add_effects_sub = sub{ $FX->apply_op };
+		# XX we should announce that will take effect in new setup
+		$add_effects_sub = $FX->is_channel_op ? sub { Audio::Nama::request_setup() } : sub{ $FX->apply_op };
 	}
-	if( Audio::Nama::valid_engine_setup() )
+	if( $this_engine->valid_setup() )
 	{
-		if (Audio::Nama::engine_running())
+		if ($this_engine->started())
 		{ 
 			$track->mute;
 			my $result = Audio::Nama::stop_do_start($add_effects_sub, 0.05);
@@ -667,7 +685,7 @@ sub insert_effect {
 	my %args = %$p;
 	local $config->{category} = 'ECI_FX';
 	return(append_effect(\%args)) if $args{before} eq 'ZZZ';
-	my $running = Audio::Nama::engine_running();
+	my $running = $this_engine->started();
 	pager("Cannot insert effect while engine is recording.\n"), return 
 		if $running and Audio::Nama::ChainSetup::really_recording();
 	pager("Cannot insert effect before controller.\n"), return 
@@ -675,7 +693,7 @@ sub insert_effect {
 	if ($running){
 		$ui->stop_heartbeat;
 		Audio::Nama::mute();
-		Audio::Nama::stop_command();
+		$this_engine->stop_command;
 		sleeper( 0.05); 
 	}
 	my $pos = fxn($args{before}) or die "$args{before}: effect ID not found";
@@ -693,8 +711,7 @@ sub insert_effect {
 
 	# remove corresponding chain operators from the engine
 	logpkg(__FILE__,__LINE__,'debug',"ops to remove and re-apply: @after_ops");
-	my $connected = Audio::Nama::eval_iam('cs-connected');
-	if ( $connected ){  
+	if ( $this_engine->valid_setup ){  
 		map{ remove_op($_)} reverse @after_ops; # reverse order for correct index
 	}
 
@@ -712,12 +729,12 @@ sub insert_effect {
 	logpkg(__FILE__,__LINE__,'debug',sub{"@{$track->ops}"});
 
 	# replace the corresponding Ecasound chain operators
-	if ($connected ){  
+	if ($this_engine->valid_setup){  
 		map{ fxn($_)->apply_op } @after_ops;
 	}
 		
 	if ($running){
-		Audio::Nama::eval_iam('start');	
+		ecasound_iam('start');	
 		sleeper(0.3);
 		Audio::Nama::unmute();
 		$ui->start_heartbeat;
@@ -725,7 +742,7 @@ sub insert_effect {
 	$added;
 }
 sub modify_effect {
-	logsub("&modify_effect");
+	logsub((caller(0))[3]);
 	my ($op_id, $parameter, $sign, $value) = @_;
 		# $parameter: one-based
 	
@@ -736,19 +753,19 @@ sub modify_effect {
 
 
 sub modify_multiple_effects {
-	logsub("&modify_multiple_effects");
+	logsub((caller(0))[3]);
 	my ($op_ids, $parameters, $sign, $value) = @_;
 	map{ my $op_id = $_;
 		map{ 	my $parameter = $_;
 				modify_effect($op_id, $parameter, $sign, $value);
 				set_current_op($op_id);
-				set_current_param($parameter);	
+				set_current_param($parameter);
 		} @$parameters;
 	} @$op_ids;
 }
 
 sub remove_effect { 
-	logsub("&remove_effect");
+	logsub((caller(0))[3]);
 	my $id = shift;
 	my $FX = fxn($id)
 		or logpkg(__FILE__,__LINE__,'logcarp',"$id: does not exist, skipping...\n"), return;
@@ -812,32 +829,36 @@ sub fx_defaults {
 ## Ecasound engine -- apply/remove chain operators
 
 sub apply_ops {  # in addition to operators in .ecs file
-	logsub("&apply_ops");
+	logsub((caller(0))[3]);
 	for my $track ( Audio::Nama::audio_tracks() ) {
 		my $n = $track->n;
  		next unless Audio::Nama::ChainSetup::is_ecasound_chain($n);
 		logpkg(__FILE__,__LINE__,'debug', "chain: $n, offset: $fx->{offset}->{$n}");
+		$this_engine->reset_ecasound_selections_cache();
+		$this_engine->current_chain($n);
 		$track->apply_ops;
 	}
-	ecasound_select_chain($this_track->n) if defined $this_track;
 }
 
 sub remove_op {
 	# remove chain operator from Ecasound engine
 
-	logsub("&remove_op");
+	logsub((caller(0))[3]);
 	local $config->{category} = 'ECI_FX';
 
 	# only if engine is configured
-	return unless Audio::Nama::valid_engine_setup();
+	return unless $this_engine->valid_setup();
 
 	my $id = shift;
 	my $self = fxn($id);
+	Audio::Nama::request_setup(), return if $self->is_channel_op;
 	my $n = $self->chain;
 
 	# select chain
 	
-	return unless ecasound_select_chain($n);
+	return unless $this_engine->valid_setup();
+	
+	$this_engine->current_chain($n);
 
 	# deal separately with controllers and chain operators
 	
@@ -850,23 +871,24 @@ sub remove_op {
 		logpkg(__FILE__,__LINE__,'debug', "ops list for chain $n: @{$ti{$n}->ops}");
 		logpkg(__FILE__,__LINE__,'debug', "operator id to remove: $id");
 		logpkg(__FILE__,__LINE__,'debug', "ready to remove from chain $n, operator id $id, index $index");
-		logpkg(__FILE__,__LINE__,'debug',sub{Audio::Nama::eval_iam("cs")});
-		Audio::Nama::eval_iam("cop-select ".  $self->ecasound_effect_index);
-		logpkg(__FILE__,__LINE__,'debug',sub{"selected operator: ". Audio::Nama::eval_iam("cop-selected")});
-		Audio::Nama::eval_iam("cop-remove");
-		logpkg(__FILE__,__LINE__,'debug',sub{Audio::Nama::eval_iam("cs")});
+		logpkg(__FILE__,__LINE__,'debug',sub{ecasound_iam("cs")});
+		$this_engine->current_chain_operator($self->ecasound_effect_index);
+		logpkg(__FILE__,__LINE__,'debug',sub{"selected operator: ". ecasound_iam("cop-selected")});
+		$this_engine->ecasound_iam("cop-remove");
+		$this_engine->reset_ecasound_selections_cache();
+		logpkg(__FILE__,__LINE__,'debug',sub{ecasound_iam("cs")});
 
 	} else { # controller
 
 		logpkg(__FILE__,__LINE__,'debug', "has parent, assuming controller");
 
 		my $ctrl_index = $self->ecasound_controller_index;
-		logpkg(__FILE__,__LINE__,'debug', Audio::Nama::eval_iam("cs"));
-		Audio::Nama::eval_iam("cop-select ".  $self->root_parent->ecasound_effect_index);
-		logpkg(__FILE__,__LINE__,'debug', "selected operator: ". Audio::Nama::eval_iam("cop-selected"));
-		Audio::Nama::eval_iam("ctrl-select $ctrl_index");
-		Audio::Nama::eval_iam("ctrl-remove");
-		logpkg(__FILE__,__LINE__,'debug', Audio::Nama::eval_iam("cs"));
+		logpkg(__FILE__,__LINE__,'debug', ecasound_iam("cs"));
+		$this_engine->current_chain_operator($self->root_parent->ecasound_controller_index);
+		logpkg(__FILE__,__LINE__,'debug', "selected operator: ". ecasound_iam("cop-selected"));
+		$this_engine->current_controller($ctrl_index);
+		$this_engine->ecasound_iam("ctrl-remove");
+		logpkg(__FILE__,__LINE__,'debug', ecasound_iam("cs"));
 	}
 }
 
@@ -911,16 +933,16 @@ sub new_effect_id {
 ## synchronize Ecasound chain operator parameters 
 #  with Nama effect parameter
 
-sub _update_effect {
+sub update_ecasound_effect {
 	local $config->{category} = 'ECI_FX';
 
 	# update the parameters of the Ecasound chain operator
 	# referred to by a Nama operator_id
 	
-	#logsub("&update_effect");
+	#logsub((caller(0))[3]);
 
-	return unless Audio::Nama::valid_engine_setup();
-	#my $es = Audio::Nama::eval_iam("engine-status");
+	return unless $this_engine->valid_setup;
+	#my $es = ecasound_iam("engine-status");
 	#logpkg(__FILE__,__LINE__,'debug', "engine is $es");
 	#return if $es !~ /not started|stopped|running/;
 
@@ -936,25 +958,23 @@ sub _update_effect {
 	# $param is zero-based. 
 	# $FX->params is  zero-based.
 
-	my $old_chain = Audio::Nama::eval_iam('c-selected') if Audio::Nama::valid_engine_setup();
-	ecasound_select_chain($chain);
+	$this_engine->current_chain($chain);
 
 	# update Ecasound's copy of the parameter
 	if( $FX->is_controller ){
 		my $i = $FX->ecasound_controller_index;
-		logpkg(__FILE__,__LINE__,'debug', "controller $id: track: $chain, index: $i param: $param, value: $val");
-		Audio::Nama::eval_iam("ctrl-select $i");
-		Audio::Nama::eval_iam("ctrlp-select $param");
-		Audio::Nama::eval_iam("ctrlp-set $val");
+		logpkg(__FILE__,__LINE__,'debug', "controller $id: track: $chain, index: $i, param: $param, value: $val");
+		$this_engine->current_controller($i);
+		$this_engine->current_controller_parameter($param);
+		$this_engine->ecasound_iam("ctrlp-set $val");
 	}
 	else { # is operator
-		my $i = $FX->ecasound_operator_index;
+		my $i = $FX->ecasound_effect_index;
 		logpkg(__FILE__,__LINE__,'debug', "operator $id: track $chain, index: $i, offset: ".  $FX->offset . " param $param, value $val");
-		Audio::Nama::eval_iam("cop-select ". ($FX->offset + $i));
-		Audio::Nama::eval_iam("copp-select $param");
-		Audio::Nama::eval_iam("copp-set $val");
+		$this_engine->current_chain_operator($i);
+		$this_engine->current_chain_operator_parameter($param);
+		$this_engine->ecasound_iam("copp-set $val");
 	}
-	ecasound_select_chain($old_chain);
 }
 
 # set both Nama effect and Ecasound chain operator
@@ -962,12 +982,13 @@ sub _update_effect {
 
 sub update_effect {
 	my ($id, $param, $val) = @_;
-	_update_effect( @_ );
 	return if ! defined fxn($id);
 	fxn($id)->params->[$param] = $val;
+	update_ecasound_effect( @_ );
 }
 
 sub sync_effect_parameters {
+	logsub((caller(0))[3]);
 	local $config->{category} = 'ECI_FX';
 
 	# when a controller changes an effect parameter, the
@@ -976,10 +997,12 @@ sub sync_effect_parameters {
 	#
 	# this routine syncs them in prep for save_state()
 	
- 	return unless Audio::Nama::valid_engine_setup();
-	my $old_chain = Audio::Nama::eval_iam('c-selected');
-	map{ $_->sync_one_effect } grep{ $_ }  map{ fxn($_) } ops_with_controller(), ops_with_read_only_params();
-	Audio::Nama::eval_iam("c-select $old_chain");
+ 	return unless $this_engine->valid_setup();
+   	push my @ops, ops_with_controller(), ops_with_read_only_params();
+	return unless @ops;
+	my $old_chain = $this_engine->current_chain;
+	map{ $_->sync_one_effect } grep{ $_ }  map{ fxn($_) } @ops;
+	$this_engine->current_chain($old_chain);
 }
 
 	
@@ -989,8 +1012,8 @@ sub get_ecasound_cop_params {
 	my $count = shift;
 	my @params;
 	for (1..$count){
-		Audio::Nama::eval_iam("copp-select $_");
-		push @params, Audio::Nama::eval_iam("copp-get");
+		$this_engine->current_chain_operator_parameter($_);
+		push @params, ecasound_iam("copp-get");
 	}
 	\@params
 }
@@ -1013,8 +1036,8 @@ sub ops_with_read_only_params {
 sub find_op_offsets {
 
 	local $config->{category} = 'ECI_FX';
-	logsub("&find_op_offsets");
-	my @op_offsets = grep{ /"\d+"/} split "\n",Audio::Nama::eval_iam("cs");
+	logsub((caller(0))[3]);
+	my @op_offsets = grep{ /"\d+"/} split "\n",ecasound_iam("cs");
 	logpkg(__FILE__,__LINE__,'debug', join "\n\n",@op_offsets);
 	for my $output (@op_offsets){
 		my $chain_id;
@@ -1077,19 +1100,20 @@ sub set_bypass_state {
 	
 	local $config->{category} = 'ECI_FX';
 	my($track, $bypass_state, @ops) = @_;
+	logsub((caller(0))[3]);
 
 	# only process ops that belong to this track
 	@ops = intersect_with_track_ops_list($track,@ops);
 
+	$this_engine->current_chain($track->n);
 	$track->mute;
-	Audio::Nama::eval_iam("c-select ".$track->n);
 
 	foreach my $op ( @ops)
 	{ 
 		my $FX = fxn($op);
 		my $i = $FX->ecasound_effect_index;
-		Audio::Nama::eval_iam("cop-select $i");
-		Audio::Nama::eval_iam("cop-bypass $bypass_state");
+		$this_engine->current_chain_operator($i);
+		$this_engine->ecasound_iam("cop-bypass $bypass_state");
 		$FX->set(bypassed => ($bypass_state eq 'on') ? 1 : 0);
 	}
 	$track->unmute;
@@ -1110,24 +1134,29 @@ sub set_current_op {
 	my $op_id = shift;
 	my $FX = fxn($op_id);
 	return unless $FX;
-	my $track = $ti{$FX->chain};
-	$project->{current_op}->{$track->name} = $op_id;
-}
-sub set_current_param {
-	my $parameter = shift;
-	$project->{current_param}->{Audio::Nama::this_op()} = $parameter;
-}
-sub set_current_stepsize {
-	my $stepsize = shift;
-	$project->{current_stepsize}->{Audio::Nama::this_op()}->[this_param()] = $stepsize;
-}
-sub increment_param { modify_effect(Audio::Nama::this_op(), this_param(),'+',this_stepsize())}
-sub decrement_param { modify_effect(Audio::Nama::this_op(), this_param(),'-',this_stepsize())}
-sub set_parameter_value {
-	my $value = shift;
-	modify_effect(Audio::Nama::this_op(), this_param(), undef, $value)
+	$project->{current_op}->{$FX->trackname} = $op_id;
 }
+sub set_current_param { $project->{current_param}->{this_op()} = $_[0] }
 
+sub this_op    	   { $this_track and $this_track->op }
+sub this_op_o 	   { $this_track and $this_track->op and fxn($this_track->op) }
+sub this_param     { $project->{current_param}->{ this_op() } }
+sub param_stepsize { $project->{param_stepsize}->{this_op() }->[ this_param() ] } 
+
+sub set_param_value {
+	#return if cannot_modify_parameter();
+	my $value = shift;
+	modify_effect(this_op(), this_param(), undef, $value)
+}
+sub increment_param {
+	return if cannot_modify_parameter();
+	my $multiplier = shift;
+	modify_effect(this_op(), this_param(), '+', $multiplier * param_stepsize())
+}
+sub increment_param_1    { increment_param(  1) }
+sub increment_param_10   { increment_param( 10) }
+sub decrement_param_1    { increment_param( -1) }
+sub decrement_param_10   { increment_param(-10) }
 
 sub check_fx_consistency {
 
@@ -1206,9 +1235,9 @@ sub fade {
 	my ($param, $from, $to, $seconds) = @_;
 
 	my $id = $self->id;
-	# no fade without Timer::HiRes
+	# no fade without Time::HiRes
 	# no fade unless engine is running
-	if ( engine_running() and $config->{hires_timer} )
+	if ( $this_engine->started() and $config->{hires_timer} )
 	{
 		my $steps = $seconds * $config->{fade_resolution};
 		my $wink  = 1/$config->{fade_resolution};
@@ -1223,6 +1252,43 @@ sub fade {
 	$self->_modify_effect($param, $to)
 }
 
+sub plan_fade {
+	return unless $this_engine->started();
+	my $self = shift;
+	my %args = @_;
+	my $param   = $args{param}   || 1;
+	my $from    = $args{from}    || 0;
+	my $to	    = $args{to}      || 0;
+	my $seconds = $args{seconds} || 1;
+	my $in_future 	= $args{in_future};
+	my $steps 	= $seconds * $config->{fade_resolution};
+	my $wink  	= 1/$config->{fade_resolution};
+	my $id 		= $self->id;
+	my $size 	= ($to - $from)/$steps;
+	logpkg(__FILE__,__LINE__,'debug', "id: $id, param: $param, from: $from, to: $to, seconds: $seconds");
+	if ( not $in_future ){
+		for (1..$steps - 1){
+			$self->_modify_effect($param, $size, '+');
+			sleeper( $wink );
+		}		
+		$self->_modify_effect($param, $to)
+	}
+	else {
+		my $advance = $in_future;
+		my $coderef = sub { $self->_modify_effect($param, $size, '+') };
+		for (1..$steps - 1){
+			$advance += $wink;
+			schedule_fade($advance, $coderef)
+		}		
+		$advance += $wink;
+		schedule_fade($advance, sub { $self->_modify_effect($param, $to) } );
+		sub schedule_fade {
+			my ($after, $sub) = @_;
+			$project->{events}->{"fade_".$setup->{fade_counter}++} = AE::timer( $after, 0, $sub );
+		}
+	}
+}
+
 sub fadein {
 	my $self = shift;
 	my $to = shift;
@@ -1247,6 +1313,20 @@ sub fade_out_level {
 	my $self = shift;
 	$config->{fade_out_level}->{$self->type}
 }
+sub ecasound_format {
+	my $self = shift;
+	my $cmd = '-'.$self->code;
+	$cmd .= ':'.join ',' ,@{$self->{params}} if $self->{params} and @{$self->{params}} > 0;
+	$cmd
+}
+# sub no_params {
+# 	my $self = shift;
+# 	$self->about->{count} == 0
+# }
+# sub cannot_modify_parameter {
+# 	$self->no_params or $self->is_read_only(current_param())
+# }
+
 
 } # end package Effect
 
