@@ -1,5 +1,5 @@
 package Audio::Nama::Globals;
-use Modern::Perl;
+use Modern::Perl '2020';
 
 # set aliases for common indices
 *bn = \%Audio::Nama::Bus::by_name;
@@ -56,6 +56,7 @@ $jack
 $fx
 $fx_cache
 $text
+$term
 $gui
 $midi
 $help
@@ -95,6 +96,7 @@ $jack
 $fx
 $fx_cache
 $text
+$term
 $gui
 $midi
 $help
@@ -162,7 +164,7 @@ $this_track_name
 
 	)],
 );
-our $ui = 'bullwinkle';  # for testing
+our $ui = 'bullwinkle';  # required for testing
 {
 	my %seen;
 	push @{$EXPORT_TAGS{all}}, grep {!$seen{$_}++} @{$EXPORT_TAGS{$_}} foreach
@@ -173,8 +175,12 @@ keys %EXPORT_TAGS;
 1;
 __END__
 
-=head1 Nama Variables
+=head1 NAME
 
+=encoding UTF-8
+ 
+Audio::Nama::Globals - Nama global variables
+ 
 Variables are listed in multiple files in the source.
 
 =head2 Exported
@@ -206,7 +212,7 @@ with object properties as need be.
 
 =over
 
-=item F<var_config>
+=item F<var_namarc>
 
 Maps keys in F<.namarc> (e.g. I<mix_to_disk_format>) to the
 corresponding Nama internal scalar (e.g. C<$config-E<gt>{mix_to_disk_format}>
@@ -216,14 +222,18 @@ corresponding Nama internal scalar (e.g. C<$config-E<gt>{mix_to_disk_format}>
 List of allowed singleton hash keys. 
 
 Keys of variables appearing in ./var_singletons 
-should be listed in var_keys or in var_config.
+should be listed in var_keys or in var_namarc.
 Undeclared keys will trigger warnings during build.
 
+=back
+
 =head2 F<var_lists>
 
 Declares lists of variables used in
 serializing/deserializing.
 
+=over
+
 =item C<@global_effect_chain_vars>
 
 Mainly user defined and system-wide effect chains,
