@@ -21,7 +21,8 @@ our (%tn, $jack, $config);
 #      + illegal track method call generate an exception
 
 package Audio::Nama::IO;
-use Modern::Perl; use Carp;
+use Modern::Perl '2020';
+use Carp;
 use Data::Dumper::Concise;
 our $VERSION = 1.0;
 
@@ -53,9 +54,11 @@ our %io_class = qw(
 	jack_ports_list_out		Audio::Nama::IO::to_jack_port
 	jack_multi_in			Audio::Nama::IO::from_jack_multi
 	jack_multi_out			Audio::Nama::IO::to_jack_multi
-	jack_client_in			Audio::Nama::IO::from_jack_client
-	jack_client_out			Audio::Nama::IO::to_jack_client
+	jack_client_in			Audio::Nama::IO::from_jack_multi
+	jack_client_out			Audio::Nama::IO::to_jack_multi
+	bus_in					Audio::Nama::IO::from_bus
 	);
+    #bus_out					Audio::Nama::IO::to_bus # 
 
 ### class descriptions
 
@@ -174,12 +177,10 @@ sub ports {
 							$client_direction,
 							$self->target_channel,
 							$self->width, 
-							Audio::Nama::try{$self->name} 
+							try{$self->name} 
 	) if $self->client
 }
 
-
-
 sub ecs_string {
 	my $self = shift;
 	my @parts;
@@ -217,10 +218,12 @@ sub AUTOLOAD {
 	return $self->$method if $self->can($method);
 	{ no warnings 'uninitialized'; 
 	if ( my $track = $tn{$self->{track_}} ){
-		return $track->$call if $track->can($call) 
+		return $track->$call if $track->can($call)
 		# ->can is reliable here because Track has no AUTOLOAD
 	}
-	# suppress error XXX
+	# XX Suppress exceptions on objects that don't have an
+	# associated track 
+	return undef if $call eq 'channel_ops';
 	return undef if $call eq 'name' or $call eq 'surname';
 	}
 	my $msg = "Autoload fell through. Object type: ". (ref $self). " illegal method call: $call\n";
@@ -302,6 +305,7 @@ sub soundcard_output_device_string {
 
 sub jack_multi_route {
 	my (@ports)  = @_;
+	@ports or return;
 	join q(,),q(jack_multi),
 	map{quote_jack_port($_)} @ports
 }
@@ -314,28 +318,33 @@ sub jack_multi_ports {
 	# can we route to these channels?
 	my $end   = $start + $width - 1;
 
-	# the following logic avoids deferencing undef for a 
-	# non-existent client, and correctly handles
-	# the case of a portname (containing colon)
-	
- 	my $channel_count = scalar @{$jack->{clients}->{$client}{$direction}};
+	my $c = client_info($client, $direction);
+	$c or return;
+ 	my $channel_count = scalar @{ $c->{$direction} };
 	my $source_or_send = $direction eq 'input' ? 'send' : 'source';
   	die(qq(
 Track $trackname: $source_or_send would cover channels $start - $end,
 out of bounds for JACK client "$client" ($channel_count channels max).
 Change $source_or_send setting, or set track OFF.)) 
 	if $end > $channel_count and $config->{enforce_channel_bounds};
-		return @{$jack->{clients}->{$client}{$direction}}[$start-1..$end-1]
-		 	if $jack->{clients}->{$client}{$direction};
+		return @{$c->{$direction}}[$start-1..$end-1]
+		 	if $c->{$direction};
 
 }
 #sub one_port { $jack->{clients}->{$client}->{$direction}->[$start-1] }
 
+sub client_info {
+	my ($client, $direction) = @_;
+ 	my $info = $jack->{clients}->{$client};
+}
+
 sub quote_jack_port {
 	my $port = shift;
+	defined $port or return;
 	($port =~ /\s/ and $port !~ /^"/) ? qq("$port") : $port
 }
 sub rectified { # client name from number
+	defined $_[0] or return;
 	$_[0] =~ /^\d+$/ 
 		? 'system'
 		: $_[0]
@@ -347,31 +356,36 @@ sub rectified { # client name from number
 
 {
 package Audio::Nama::IO::from_null;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub _device_id { 'null' }  
 }
 
 {
 package Audio::Nama::IO::to_null;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub _device_id { 'null' }
 }
 
 {
 package Audio::Nama::IO::from_rtnull;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub _device_id { 'rtnull' }  
 }
 
 {
 package Audio::Nama::IO::to_rtnull;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub _device_id { 'rtnull' }  
 }
 
 {
 package Audio::Nama::IO::from_wav;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub device_id { 
 	my $self = shift;
 	my @modifiers;
@@ -381,35 +395,45 @@ sub device_id {
 	push @modifiers, $self->full_path;
 	join(q[,],@modifiers);
 }
+sub _format { $Audio::Nama::setup->{wav_info}->{$_[0]->full_path}->{format} };
 sub ecs_extra { $_[0]->mono_to_stereo}
-sub client { 'system' if $jack->{jackd_running} } # since we share latency value
+sub client { 'system' if $jack->{jackd_running} } 
 sub ports { 'system:capture_1' }
 }
 {
 package Audio::Nama::IO::to_wav;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub device_id { $_[0]->full_path }
 sub _format_template { $config->{raw_to_disk_format} } 
 }
 
 {
 package Audio::Nama::IO::from_loop;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub new {
 	my $class = shift;
 	my %vals = @_;
 	$class->SUPER::new( %vals, device_id => "loop,$vals{endpoint}");
 }
-sub format {}
+sub _format { 
+	my $self = shift;
+	return if $Audio::Nama::config->{opts}->{T}; # XX don't break tests
+	Audio::Nama::signal_format($self->format_template, $config->{loop_chain_channel_width});
+}
+sub _format_template { $config->{cache_to_disk_format} } 
 }
 {
 package Audio::Nama::IO::to_loop;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO::from_loop';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO::from_loop';
 }
 
 {
 package Audio::Nama::IO::from_soundcard;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub new {
 	shift; # throw away class
 	my $class = $io_class{Audio::Nama::IO::soundcard_input_type_string()};
@@ -418,7 +442,8 @@ sub new {
 }
 {
 package Audio::Nama::IO::to_soundcard;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub new {
 	shift; # throw away class
 	my $class = $io_class{Audio::Nama::IO::soundcard_output_type_string()};
@@ -427,7 +452,8 @@ sub new {
 }
 {
 package Audio::Nama::IO::to_jack_multi;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub client { 
 	my $self = shift;
 #  	say "to_jack_multi: target_id: ",$self->target_id;
@@ -442,38 +468,41 @@ sub device_id {
 
 {
 package Audio::Nama::IO::from_jack_multi;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO::to_jack_multi';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO::to_jack_multi';
 sub ecs_extra { $_[0]->mono_to_stereo }
 }
 
 {
 package Audio::Nama::IO::to_jack_port;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub format_template { $config->{devices}->{jack}->{signal_format} }
 sub device_id { 'jack,,'.$_[0]->port_name.'_out' }
-sub ports { "Nama:".$_[0]->port_name. '_out_1' } # at least this one port
-	# HARDCODED port name
+sub ports { $config->{ecasound_jack_client_name}. ":".$_[0]->port_name. '_out_1' } # at least this one port
 }
 
 {
 package Audio::Nama::IO::from_jack_port;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO::to_jack_port';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO::to_jack_port';
 sub device_id { 'jack,,'.$_[0]->port_name.'_in' }
 sub ecs_extra { $_[0]->mono_to_stereo }
-sub ports { "Nama:".$_[0]->port_name. '_in_1' } # at least this one port
-	# HARDCODED port name
+sub ports { $config->{ecasound_jack_client_name}.":".$_[0]->port_name. '_in_1' } # at least this one port
 }
 
 {
 package Audio::Nama::IO::to_jack_client;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub device_id { "jack," . Audio::Nama::IO::quote_jack_port($_[0]->send_id); }
 sub client { Audio::Nama::IO::rectified($_[0]->send_id) }
 }
 
 {
 package Audio::Nama::IO::from_jack_client;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub device_id { 'jack,'.  Audio::Nama::IO::quote_jack_port($_[0]->source_id); }
 sub ecs_extra { $_[0]->mono_to_stereo}
 sub client { Audio::Nama::IO::rectified($_[0]->source_id) }
@@ -481,7 +510,8 @@ sub client { Audio::Nama::IO::rectified($_[0]->source_id) }
 
 {
 package Audio::Nama::IO::from_alsa_soundcard_device;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub ecs_extra { join ' ', $_[0]->rec_route, $_[0]->mono_to_stereo }
 sub device_id { $config->{devices}->{$config->{alsa_capture_device}}->{ecasound_id} }
 sub input_channel { $_[0]->source_id }
@@ -501,7 +531,8 @@ sub rec_route {
 }
 {
 package Audio::Nama::IO::to_alsa_soundcard_device;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 sub device_id { $config->{devices}->{$config->{alsa_playback_device}}{ecasound_id} }
 sub ecs_extra {route($_[0]->width,$_[0]->output_channel) }
 sub output_channel { $_[0]->send_id }
@@ -523,8 +554,20 @@ sub route {
 }
 }
 {
+package Audio::Nama::IO::from_bus;
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
+sub new {
+	my $class = shift;
+	my %vals = @_;
+	print "from_bus: ", Audio::Nama::Dumper \%vals;
+	#$class->SUPER::new( %vals, device_id => "loop,$vals{endpoint}");
+}
+}
+{
 package Audio::Nama::IO::any;
-use Modern::Perl; use vars qw(@ISA); @ISA = 'Audio::Nama::IO';
+use Modern::Perl '2020';
+our @ISA = 'Audio::Nama::IO';
 }
 
 
