@@ -1,7 +1,7 @@
 # ------- WAV file info routines ---------
 
 package Audio::Nama;
-use Modern::Perl;
+use Modern::Perl '2020';
 
 ### WAV file length/format/modify_time are cached in $setup->{wav_info} 
 
@@ -50,8 +50,10 @@ sub _get_modify_time {
 }
 sub _update_wav_cache {
 	my $path = shift;
+	{ no warnings 'uninitialized';
 	return unless _get_modify_time($path) != $setup->{wav_info}->{$path}{modify_time};
-	say qq(WAV file $path has changed! Updating cache.);
+	}
+	# throw("WAV file $path has changed! Updating cache.");
 	_get_wav_info($path) 
 }
 
@@ -63,15 +65,17 @@ sub ecasound_get_info {
 	local $config->{category} = 'ECI_WAVINFO';
 
 	$path = qq("$path");
+	my $old = ecasound_iam('cs-selected');
 	teardown_engine();
-	eval_iam('cs-add gl');
-	eval_iam('c-add g');
-	eval_iam('ai-add ' . $path);
-	eval_iam('ao-add null');
-	eval_iam('cs-connect');
-	eval_iam('ai-select '. $path);
-	my $result = eval_iam($command);
+	ecasound_iam('cs-add gl');
+	ecasound_iam('c-add g');
+	ecasound_iam('ai-add ' . $path);
+	ecasound_iam('ao-add null');
+	ecasound_iam('cs-connect');
+	ecasound_iam('ai-select '. $path);
+	my $result = ecasound_iam($command);
 	teardown_engine();
+	ecasound_iam("cs-load $old") if $old;
 	$result;
 }
 1;
